#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <gtk/gtk.h>
#include "3dfx_tvtool.h"
#include "3dfx_tvtool_help.h"

int main(int argc, char *argv[]) {
	GtkWidget *window;
	GtkWidget *button;
	GtkWidget *main_box;
	char data[DATA_SIZE];
	
	gtk_init(&argc, &argv);

	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
		gtk_window_set_title(GTK_WINDOW(window), (gpointer)"3dfx TV Tool v"VERSION);
		g_signal_connect(G_OBJECT(window), "destroy", G_CALLBACK(gtk_main_quit), NULL);
		g_signal_connect(G_OBJECT(window), "delete_event", G_CALLBACK(gtk_main_quit), NULL);
		gtk_container_set_border_width(GTK_CONTAINER(window), 15);
		
		main_box = gtk_hbox_new(TRUE, 0);
			gtk_container_add(GTK_CONTAINER(window), main_box);
		
			if(!read_tv_out(COLORBARS, data)) {
				button = gtk_toggle_button_new();
					g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(button_action), (gpointer)"COLORBARS");
					gtk_box_pack_start(GTK_BOX(main_box), button, TRUE, TRUE, 0);
					if(strcmp(data,"1") > 0) {
						gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), TRUE);
						gtk_button_set_label(GTK_BUTTON(button), (gpointer)COLORBARS_BUTTON_ON);
						gtk_button_set_use_underline(GTK_BUTTON(button), TRUE);
					} else {
						gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), FALSE);
						gtk_button_set_label(GTK_BUTTON(button), (gpointer)COLORBARS_BUTTON_OFF);
						gtk_button_set_use_underline(GTK_BUTTON(button), TRUE);
						
					}
				gtk_widget_show(button);
			}
			
			if(!read_tv_out(TV_OFF, data)) {
				button = gtk_toggle_button_new();
					g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(button_action), (gpointer)"TV_OFF");
					gtk_box_pack_start(GTK_BOX(main_box), button, TRUE, TRUE, 0);
					if(strcmp(data,"1") > 0) {
						gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), FALSE);
						gtk_button_set_label(GTK_BUTTON(button), (gpointer)TV_OUT_DISABLED);
						gtk_button_set_use_underline(GTK_BUTTON(button), TRUE);
					} else {
						gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), TRUE);
						gtk_button_set_label(GTK_BUTTON(button), (gpointer)TV_OUT_ENABLED);
						gtk_button_set_use_underline(GTK_BUTTON(button), TRUE);
					}
				gtk_widget_show(button);
			}
			
			if(!read_tv_out(NTSC, data)) {
				button = gtk_toggle_button_new();
					g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(button_action), (gpointer)"NTSC");
					gtk_box_pack_start(GTK_BOX(main_box), button, TRUE, TRUE, 0);
					if(strcmp(data,"1") > 0) {
						gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), TRUE);
						gtk_button_set_label(GTK_BUTTON(button), (gpointer)MODE_NTSC);
						gtk_button_set_use_underline(GTK_BUTTON(button), TRUE);
					} else {
						gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), FALSE);
						gtk_button_set_label(GTK_BUTTON(button), (gpointer)MODE_PAL);
						gtk_button_set_use_underline(GTK_BUTTON(button), TRUE);
				}
				gtk_widget_show(button);
			}

			if(!read_tv_out(STATUS, data)) {
				button = gtk_button_new_with_mnemonic(REAPPLY);
					g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(button_action), (gpointer)"STATUS");
					gtk_box_pack_start(GTK_BOX(main_box), button, TRUE, TRUE, 0);
				gtk_widget_show(button);
			}
				
			button = gtk_button_new_with_mnemonic(QUICK_SAVE);
				g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(button_action), (gpointer)"SAVE");
				gtk_box_pack_start(GTK_BOX(main_box), button, TRUE, TRUE, 0);
				gtk_widget_show(button);
		
			button = gtk_button_new_with_mnemonic(QUICK_LOAD);
				g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(button_action), (gpointer)"LOAD");
				gtk_box_pack_start(GTK_BOX(main_box), button, TRUE, TRUE, 0);
				gtk_widget_show(button);

			button = gtk_button_new_with_label(QUIT);
				g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(gtk_main_quit), NULL);
				gtk_box_pack_start(GTK_BOX(main_box), button, TRUE, TRUE, 0);
			gtk_widget_show(button);
					
		gtk_widget_show(main_box);	
	gtk_widget_show(window);
		
	gtk_main();
        return 0;
}

int write_tv_out(char *control, char *data) {
        FILE *control_file;
        control_file = fopen(control,"w");
        if(control_file) {
		fprintf(control_file, "%s", data);
        	fclose(control_file);
#ifdef DEBUG
		g_print(WRITE_TV_OUT_WRITTEN, data, control);
#endif
        } else {
                fprintf(stderr,WRITE_TV_OUT_COULD_NOT_WRITE, data, control);
		return 1;
        }
        return 0;
}

int read_tv_out(char *control, char *data) {
	FILE *control_file;
	control_file = fopen(control,"r");
	if(control_file) {
		fgets(data, DATA_SIZE, control_file);
	} else {
		fprintf(stderr,ERROR_READING_FILE, control);
		return 1;
	}
	return 0;
}

int load_settings(char *filename) {
	FILE *load_file;
	char data[127], *data_ptr;
	load_file = fopen(filename,"r");
	if(load_file) {
		fgets(data,sizeof(data),load_file);
		if(!strcmp(data,"v"VERSION)) {
			fputs(LOAD_SETTINGS_WRONG_VERSION,stderr);
		} else {
			g_print("Loading ...\n");
			fgets(data,sizeof(data),load_file);
			data_ptr = data;
			strsep(&data_ptr,"\t");
			write_tv_out(CHROMA,strsep(&data_ptr,"\0"));
			fgets(data,sizeof(data),load_file);
			data_ptr = data;
			strsep(&data_ptr,"\t");
			write_tv_out(COLORBARS,strsep(&data_ptr,"\0"));
			fgets(data,sizeof(data),load_file);
			data_ptr = data;
			strsep(&data_ptr,"\t");
			write_tv_out(DEPTH,strsep(&data_ptr,"\0"));
			fgets(data,sizeof(data),load_file);
			data_ptr = data;
			strsep(&data_ptr,"\t");
			write_tv_out(HALF,strsep(&data_ptr,"\0"));
			fgets(data,sizeof(data),load_file);
			data_ptr = data;
			strsep(&data_ptr,"\t");
			write_tv_out(INTERLACE,strsep(&data_ptr,"\0"));
			fgets(data,sizeof(data),load_file);
			data_ptr = data;
			strsep(&data_ptr,"\t");
			write_tv_out(LUMA,strsep(&data_ptr,"\0"));
			fgets(data,sizeof(data),load_file);
			data_ptr = data;
			strsep(&data_ptr,"\t");
			write_tv_out(NTSC,strsep(&data_ptr,"\0"));
			fgets(data,sizeof(data),load_file);
			data_ptr = data;
			strsep(&data_ptr,"\t");
			write_tv_out(RES,strsep(&data_ptr,"\0"));
			fgets(data,sizeof(data),load_file);
			data_ptr = data;
			strsep(&data_ptr,"\t");
			write_tv_out(HALF,strsep(&data_ptr,"\0"));
			fgets(data,sizeof(data),load_file);
			data_ptr = data;
			strsep(&data_ptr,"\t");
			write_tv_out(TV_OFF,strsep(&data_ptr,"\0"));
		}
		fclose(load_file);
	} else {
		fprintf(stderr,ERROR_READING_FILE, filename);
		return 1;
	}
	return 0;
}

int save_settings(char *filename) {
	FILE *save_file;
	char data[DATA_SIZE];
	save_file = fopen(filename,"w");
	if(save_file) {
		fprintf(save_file,SAVEFILE_HEADER, VERSION);
		read_tv_out(CHROMA, data);
		fprintf(save_file,"Chroma\t%s", data);
		read_tv_out(COLORBARS, data);
		fprintf(save_file,"Colorbars\t%s", data);
		read_tv_out(DEPTH, data);
		fprintf(save_file,"Depth\t%s", data);
		read_tv_out(INTERLACE, data);
		fprintf(save_file,"Half\t%s", data);
		read_tv_out(LUMA, data);
		fprintf(save_file,"Interlace\t%s", data);
		read_tv_out(NTSC, data);
		fprintf(save_file,"Luma\t%s", data);
		read_tv_out(SVIDEO, data);
		fprintf(save_file,"NTSC\t%s", data);
		read_tv_out(RES, data);
		fprintf(save_file,"Res\t%s", data);
		read_tv_out(HALF, data);
		fprintf(save_file,"SVideo\t%s", data);
		read_tv_out(TV_OFF, data);
		fprintf(save_file,"TVOff\t%s", data);
		fclose(save_file);
	} else {
		fprintf(stderr,ERROR_WRITING_FILE, filename);
		return 1;
	}
	return 0;
}

int button_action(GtkWidget *button, gpointer data) {
	char dummy[DATA_SIZE];
	if(data=="COLORBARS") {	
		if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(button))) {
			gtk_button_set_label(GTK_BUTTON(button), (gpointer)COLORBARS_BUTTON_ON);
			gtk_button_set_use_underline(GTK_BUTTON(button), TRUE);
			write_tv_out(COLORBARS, "1");
    		} else {
			gtk_button_set_label(GTK_BUTTON(button), (gpointer)COLORBARS_BUTTON_OFF);
			write_tv_out(COLORBARS, "0");
		}
	}
	if(data=="TV_OFF") {
		if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(button))) {
			gtk_button_set_label(GTK_BUTTON(button), (gpointer)TV_OUT_DISABLED);
			write_tv_out(TV_OFF, "0");
		} else {
			gtk_button_set_label(GTK_BUTTON(button), (gpointer)TV_OUT_ENABLED);
			write_tv_out(TV_OFF, "1");
		}
	}
	if(data=="NTSC") {
		if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(button))) {
			gtk_button_set_label(GTK_BUTTON(button), (gpointer)MODE_NTSC);
			write_tv_out(NTSC, "1");
		} else {
			gtk_button_set_label(GTK_BUTTON(button), (gpointer)MODE_PAL);
			write_tv_out(NTSC, "0");
		}
	}
	if(data=="STATUS") {
		read_tv_out(STATUS, dummy);
	}
	if(data=="SAVE") {
		save_settings(DEFAULT_SAVE_FILE);
	}
	if(data=="LOAD") {
		load_settings(DEFAULT_SAVE_FILE);
	}
}
